"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizePath = exports.Index = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@yarnpkg/core");
const toPath_1 = tslib_1.__importDefault(require("lodash/toPath"));
class Index {
    constructor(indexedFields) {
        this.indexedFields = indexedFields;
        this.items = [];
        this.indexes = {};
        this.clear();
    }
    clear() {
        this.items = [];
        for (const field of this.indexedFields) {
            this.indexes[field] = new Map();
        }
    }
    insert(item) {
        this.items.push(item);
        for (const field of this.indexedFields) {
            const value = Object.prototype.hasOwnProperty.call(item, field)
                ? item[field]
                : undefined;
            if (typeof value === `undefined`)
                continue;
            const list = core_1.miscUtils.getArrayWithDefault(this.indexes[field], value);
            list.push(item);
        }
        return item;
    }
    find(filter) {
        var _a;
        if (typeof filter === `undefined`)
            return this.items;
        const filterEntries = Object.entries(filter);
        if (filterEntries.length === 0)
            return this.items;
        const sequentialFilters = [];
        let matches;
        for (const [field_, value] of filterEntries) {
            const field = field_;
            const index = Object.prototype.hasOwnProperty.call(this.indexes, field)
                ? this.indexes[field]
                : undefined;
            if (typeof index === `undefined`) {
                sequentialFilters.push([field, value]);
                continue;
            }
            const filterMatches = new Set((_a = index.get(value)) !== null && _a !== void 0 ? _a : []);
            if (filterMatches.size === 0)
                return [];
            if (typeof matches === `undefined`) {
                matches = filterMatches;
            }
            else {
                for (const item of matches) {
                    if (!filterMatches.has(item)) {
                        matches.delete(item);
                    }
                }
            }
            if (matches.size === 0) {
                break;
            }
        }
        let result = [...matches !== null && matches !== void 0 ? matches : []];
        if (sequentialFilters.length > 0) {
            result = result.filter(item => {
                for (const [field, value] of sequentialFilters) {
                    const valid = typeof value !== `undefined`
                        ? Object.prototype.hasOwnProperty.call(item, field) && item[field] === value
                        : Object.prototype.hasOwnProperty.call(item, field) === false;
                    if (!valid) {
                        return false;
                    }
                }
                return true;
            });
        }
        return result;
    }
}
exports.Index = Index;
const numberRegExp = /^[0-9]+$/;
const identifierRegExp = /^[a-zA-Z0-9_]+$/;
const knownDictKeys = new Set([`scripts`, ...core_1.Manifest.allDependencies]);
function isKnownDict(parts, index) {
    return index === 1 && knownDictKeys.has(parts[0]);
}
function normalizePath(p) {
    const parts = Array.isArray(p)
        ? p
        : (0, toPath_1.default)(p);
    const normalizedParts = parts.map((part, t) => {
        if (numberRegExp.test(part))
            return `[${part}]`;
        if (identifierRegExp.test(part) && !isKnownDict(parts, t))
            return `.${part}`;
        return `[${JSON.stringify(part)}]`;
    });
    return normalizedParts.join(``).replace(/^\./, ``);
}
exports.normalizePath = normalizePath;
